@@ -16,11 +16,16 @@ class Canvas:
         self.__hist.append({})
         # frames to export as a GIF file
         self.__frames = []
+        # flip canvas
+        self.__parameters()['flip_x']          = False
+        self.__parameters()['flip_y']          = False
         # canvas origin position
         self.__parameters()['center_x']        = 0
         self.__parameters()['center_y']        = 0
         # rotation degree
         self.__parameters()['rotate_deg']      = 0
+        # zoom value
+        self.__parameters()['zoom']            = 1
         # fill and stroke color
         self.__parameters()['fill_color']      = 'white'
         self.__parameters()['stroke_color']    = 'black'
@@ -29,6 +34,7 @@ class Canvas:
         self.__parameters()['stroke_disabled'] = False
         # vertices data to create polygon
         self.__vertices        = []
+
         self.__width           = kwargs.get('width', 0)
         self.__height          = kwargs.get('height', 0)
         # list of shapes drawn on PhotoImage
@@ -36,6 +42,8 @@ class Canvas:
         # font styles
         self.__parameters()['font_family']     = 'Tahoma 20'
         self.__parameters()['font_color']      = 'black'
+        # text object anchor point
+        self.__parameters()['text_anchor']     = 'center'
 
         # tkinter's default methods are available in handle
         self.handle = tk.Canvas(master,
@@ -103,6 +111,12 @@ class Canvas:
         sin_val = math.sin(angle)
         new_coords = []
         for x_old, y_old in coords:
+            # check if canvas is flipped
+            if self.__parameters()['flip_y']:
+                y_old = - y_old
+            if self.__parameters()['flip_x']:
+                x_old = - x_old
+
             x_new = (x_old * cos_val - y_old * sin_val)
             y_new = (x_old * sin_val + y_old * cos_val)
 
@@ -110,6 +124,10 @@ class Canvas:
                 x_new *= self.app.scale_x
                 y_new *= -self.app.scale_y
 
+            # set zoom
+            x_new *= self.__parameters()['zoom']
+            y_new *= self.__parameters()['zoom']
+
             new_coords.append((x_new + cx, y_new + cy))
 
         return new_coords
@@ -125,10 +143,25 @@ class Canvas:
         self.__parameters()['center_x'] = x
         self.__parameters()['center_y'] = y
 
+    # flip canvas
+    def flip(self, direction):
+        dir = direction.lower()
+        if any(c not in 'xy' for c in dir):
+            raise ValueError('The parameter must be either "X" or "Y".')
+
+        if 'x' in dir:
+            self.__parameters()['flip_x'] = not self.__parameters()['flip_x']
+        if 'y' in dir:
+            self.__parameters()['flip_y'] = not self.__parameters()['flip_y']
+
     # set rotation value
     def rotate(self, deg):
         self.__parameters()['rotate_deg'] = deg
 
+    # set zoom value
+    def zoom(self, scale):
+        self.__parameters()['zoom'] = scale
+
     # set fill color
     def fill(self, color):
         self.__parameters()['fill_color'] = color
@@ -160,9 +193,9 @@ class Canvas:
         cy = (y1 + y2) // 2
 
         if self.app.useBounds:
-            size = ((radius * 2 * self.app.scale_x), (radius * 2 * self.app.scale_y))
+            size = ((radius * 2 * self.app.scale_x) + 1, (radius * 2 * self.app.scale_y) + 1)
         else:
-            size = ((radius * 2), (radius * 2))
+            size = ((radius * 2) + 1, (radius * 2) + 1)
 
         alpha = int(kwargs.get('alpha', 1) * 255)
         bgAlpha = alpha
@@ -234,7 +267,7 @@ class Canvas:
                                         width = self.__parameters()['stroke_width'])
     @dispatch(Vector.Vector, Vector.Vector)
     def line(self, v1, v2):
-        return self.line(int(v1.a), int(v1.b), int(v2.a), int(v2.b))
+        return self.line(int(v1.x), int(v1.y), int(v2.x), int(v2.y))
 
     # draw triangle on canvas using customShape
     @dispatch(object, object, object, object, object, object)
@@ -249,9 +282,9 @@ class Canvas:
         return obj
     @dispatch(Vector.Vector, Vector.Vector, Vector.Vector)
     def triangle(self, v1, v2, v3):
-        return self.triangle(v1.a, v1.b,
-                             v2.a, v2.b,
-                             v3.a, v3.b)
+        return self.triangle(v1.x, v1.y,
+                             v2.x, v2.y,
+                             v3.x, v3.y)
 
     # draw arc on canvas
     def arc(self, x1, y1, x2, y2, start, extend):
@@ -324,12 +357,18 @@ class Canvas:
     def font_color(self, color):
         self.__parameters()['font_color'] = color
 
+    # set text anchor point
+    # nw, n, ne, center, sw, s, se
+    def text_anchor(self, anchor):
+        self.__parameters()['text_anchor'] = anchor
+
     # put text on canvas
     def text(self, x, y, text):
         return self.handle.create_text(self.transform_coords([[x, y]]),
                                        fill = self.__parameters()['font_color'],
                                        font = self.__parameters()['font_family'],
                                        angle = -self.__parameters()['rotate_deg'],
+                                       anchor = self.__parameters()['text_anchor'],
                                        text = text)
 
     # set a pixel
@@ -353,6 +392,8 @@ class Canvas:
         except:
             return (0, 0, 0)
 
+    # load an image from path and draw it on canvas
+    @dispatch(object, object, object)
     def create_image(self, x, y, path, **kwargs):
         im = Image.open(path)
 
@@ -367,6 +408,9 @@ class Canvas:
         return self.handle.create_image(self.transform_coords([[x, y]]),
                                         image = self.__alpha_shapes[-1],
                                         anchor = 'center')
+    @dispatch(Vector.Vector, object)
+    def create_image(self, v, path, **kwargs):
+        return self.create_image(v.x, v.y, path, **kwargs)
 
     # return the latest parameters
     def __parameters(self):
