@@ -12,11 +12,13 @@ import tkinter as tk
 from EasyDraw import Color
 from EasyDraw import Canvas
 from EasyDraw import Tools
+from EasyDraw import Vector
 
 class EasyDraw(object):
     '''EasyDraw main class'''
     def __init__(self, **kwargs):
         print('Hello from EasyDraw!')
+        self.tick         = 1
         self.width        = kwargs.get('width', 400)
         self.height       = kwargs.get('height', 400)
         self.autoClear    = kwargs.get('autoClear', True)  
@@ -67,7 +69,8 @@ class EasyDraw(object):
         self.color = Color.Color()
 
         self.setupFunction  = kwargs.get('setupFunc', None)
-        self.drawFunction   = kwargs.get('drawFunc', None)
+        self.drawFunction   = kwargs.get('drawFunc', None)  
+
         self.mouseMoveFunction = kwargs.get('mouseMoveFunc', None)
         self.clickFunction = kwargs.get('clickFunc', None)
         self.mouseDownFunction = kwargs.get('mouseDownFunc', None)
@@ -114,9 +117,8 @@ class EasyDraw(object):
     '''mouse related methods ---------------------------------------'''
     def __get_mouse_positions(self, e):
         if self.useBounds:
-            transformed = self.canvas.transform_coords([[e.x, e.y]])
-            self.mouse_x = transformed[0][0]
-            self.mouse_y = transformed[0][1]
+            self.mouse_x = e.x
+            self.mouse_y = e.y
             self.mouse_x = self.tools.map(self.mouse_x,
                                           0,
                                           self.width,
@@ -135,7 +137,6 @@ class EasyDraw(object):
         self.mouse_top = e.y
 
     def __motion_event(self, event):
-        self.__get_mouse_positions(event)
         if self.mouseMoveFunction:
             self.mouseMoveFunction(self)
 
@@ -152,12 +153,10 @@ class EasyDraw(object):
             self.clickFunction(self, 'right')
 
     def __left_mouse_btn_down(self, event):
-        self.__get_mouse_positions(event)
         if self.mouseDownFunction:
             self.mouseDownFunction(self, 'left')
 
     def __right_mouse_btn_down(self, event):
-        self.__get_mouse_positions(event)
         if self.mouseDownFunction:
             self.mouseDownFunction(self, 'right')
 
@@ -183,6 +182,11 @@ class EasyDraw(object):
             self.canvas.clear_data()
 
         if callable(self.drawFunction):
+            self.tick += 1
+            mx = self.master.winfo_pointerx() - self.master.winfo_rootx()
+            my = self.master.winfo_pointery() - self.master.winfo_rooty()
+            if mx > 0 and mx <= self.width and my > 0 and my <= self.height:
+                self.__get_mouse_positions(Vector.Vector(mx, my))
             self.drawFunction(self)
         else:
             raise Exception('Draw function is either undefined or not callable!')
